@@ -1,18 +1,20 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright 2021 Keylime Authors
 
-use crate::crypto;
 use crate::{
-    common::{
-        AuthTag, EncryptedData, JsonWrapper, KeySet, SymmKey, AGENT_UUID_LEN,
-        AUTH_TAG_LEN,
-    },
+    common::JsonWrapper,
     config::KeylimeConfig,
     payloads::{Payload, PayloadMessage},
     Error, QuoteData, Result,
 };
-use actix_web::{web, HttpRequest, HttpResponse, Responder};
+use actix_web::{http, web, HttpRequest, HttpResponse, Responder};
 use base64::{engine::general_purpose, Engine as _};
+use keylime::crypto::{
+    self,
+    auth_tag::AuthTag,
+    encrypted_data::EncryptedData,
+    symmkey::{KeySet, SymmKey},
+};
 use log::*;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
@@ -130,10 +132,10 @@ fn try_combine_keys(
     None
 }
 
-pub(crate) async fn u_key(
+async fn u_key(
     body: web::Json<KeylimeUKey>,
     req: HttpRequest,
-    quote_data: web::Data<QuoteData>,
+    quote_data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     debug!("Received ukey");
 
@@ -244,10 +246,10 @@ pub(crate) async fn u_key(
     HttpResponse::Ok().json(JsonWrapper::success(()))
 }
 
-pub(crate) async fn v_key(
+async fn v_key(
     body: web::Json<KeylimeVKey>,
     req: HttpRequest,
-    quote_data: web::Data<QuoteData>,
+    quote_data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     debug!("Received vkey");
 
@@ -315,9 +317,9 @@ pub(crate) async fn v_key(
     HttpResponse::Ok().json(JsonWrapper::success(()))
 }
 
-pub(crate) async fn pubkey(
+async fn pubkey(
     req: HttpRequest,
-    data: web::Data<QuoteData>,
+    data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     match crypto::pkey_pub_to_pem(&data.pub_key) {
         Ok(pubkey) => {
@@ -364,10 +366,10 @@ async fn get_symm_key(
     }
 }
 
-pub(crate) async fn verify(
+async fn verify(
     param: web::Query<KeylimeChallenge>,
     req: HttpRequest,
-    data: web::Data<QuoteData>,
+    data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     if param.challenge.is_empty() {
         warn!(
@@ -544,6 +546,57 @@ pub(crate) async fn worker(
     Ok(())
 }
 
+/// Handles the default case for the /keys scope
+async fn keys_default(req: HttpRequest) -> impl Responder {
+    let error;
+    let response;
+    let message;
+
+    match req.head().method {
+        http::Method::GET => {
+            error = 400;
+            message = "URI not supported, only /pubkey and /verify are supported for GET in /keys interface";
+            response = HttpResponse::BadRequest()
+                .json(JsonWrapper::error(error, message));
+        }
+        http::Method::POST => {
+            error = 400;
+            message = "URI not supported, only /ukey and /vkey are supported for POST in /keys interface";
+            response = HttpResponse::BadRequest()
+                .json(JsonWrapper::error(error, message));
+        }
+        _ => {
+            error = 405;
+            message = "Method is not supported in /keys interface";
+            response = HttpResponse::MethodNotAllowed()
+                .insert_header(http::header::Allow(vec![
+                    http::Method::GET,
+                    http::Method::POST,
+                ]))
+                .json(JsonWrapper::error(error, message));
+        }
+    };
+
+    warn!(
+        "{} returning {} response. {}",
+        req.head().method,
+        error,
+        message
+    );
+
+    response
+}
+
+/// Configure the endpoints for the /keys scope
+pub(crate) fn configure_keys_endpoints(cfg: &mut web::ServiceConfig) {
+    _ = cfg
+        .service(web::resource("/pubkey").route(web::get().to(pubkey)))
+        .service(web::resource("/ukey").route(web::post().to(u_key)))
+        .service(web::resource("/verify").route(web::get().to(verify)))
+        .service(web::resource("/vkey").route(web::post().to(v_key)))
+        .default_service(web::to(keys_default));
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -552,7 +605,6 @@ mod tests {
         encrypt_aead, pkey_pub_from_pem, rsa_oaep_encrypt,
     };
     use crate::{
-        common::API_VERSION,
         config::KeylimeConfig,
         crypto::{compute_hmac, AES_128_KEY_LEN, AES_256_KEY_LEN},
         payloads,
@@ -567,6 +619,7 @@ mod tests {
         rsa::Padding,
         sign::Signer,
     };
+    use serde_json::{json, Value};
     use std::{
         env, fs,
         path::{Path, PathBuf},
@@ -771,10 +824,59 @@ mod tests {
         }
     }
 
+    #[actix_rt::test]
+    async fn test_keys_default() {
+        let mut app = test::init_service(
+            App::new().service(web::resource("/").to(keys_default)),
+        )
+        .await;
+
+        let req = test::TestRequest::get().uri("/").to_request();
+
+        let resp = test::call_service(&app, req).await;
+        assert!(resp.status().is_client_error());
+
+        let result: JsonWrapper<Value> = test::read_body_json(resp).await;
+
+        assert_eq!(result.results, json!({}));
+        assert_eq!(result.code, 400);
+
+        let req = test::TestRequest::post()
+            .uri("/")
+            .data("some data")
+            .to_request();
+
+        let resp = test::call_service(&app, req).await;
+        assert!(resp.status().is_client_error());
+
+        let result: JsonWrapper<Value> = test::read_body_json(resp).await;
+
+        assert_eq!(result.results, json!({}));
+        assert_eq!(result.code, 400);
+
+        let req = test::TestRequest::delete().uri("/").to_request();
+
+        let resp = test::call_service(&app, req).await;
+        assert!(resp.status().is_client_error());
+
+        let headers = resp.headers();
+
+        assert!(headers.contains_key("allow"));
+        assert_eq!(
+            headers.get("allow").unwrap().to_str().unwrap(), //#[allow_ci]
+            "GET, POST"
+        );
+
+        let result: JsonWrapper<Value> = test::read_body_json(resp).await;
+
+        assert_eq!(result.results, json!({}));
+        assert_eq!(result.code, 405);
+    }
+
     #[cfg(feature = "testing")]
     async fn test_u_or_v_key(key_len: usize, payload: Option<&[u8]>) {
         let test_config = KeylimeConfig::default();
-        let mut fixture = QuoteData::fixture().unwrap(); //#[allow_ci]
+        let (mut fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
 
         // Create temporary working directory and secure mount
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
@@ -801,18 +903,9 @@ mod tests {
         let mut app = test::init_service(
             App::new()
                 .app_data(quotedata.clone())
-                .route(
-                    &format!("/{API_VERSION}/keys/ukey"),
-                    web::post().to(u_key),
-                )
-                .route(
-                    &format!("/{API_VERSION}/keys/vkey"),
-                    web::post().to(v_key),
-                )
-                .route(
-                    &format!("/{API_VERSION}/keys/verify"),
-                    web::get().to(verify),
-                ),
+                .route("/vX.Y/keys/ukey", web::post().to(u_key))
+                .route("/vX.Y/keys/vkey", web::post().to(v_key))
+                .route("/vX.Y/keys/verify", web::get().to(verify)),
         )
         .await;
 
@@ -879,7 +972,7 @@ mod tests {
         };
 
         let req = test::TestRequest::post()
-            .uri(&format!("/{API_VERSION}/keys/ukey"))
+            .uri("/vX.Y/keys/ukey")
             .set_json(&ukey)
             .to_request();
 
@@ -894,7 +987,7 @@ mod tests {
         };
 
         let req = test::TestRequest::post()
-            .uri(&format!("/{API_VERSION}/keys/vkey"))
+            .uri("/vX.Y/keys/vkey")
             .set_json(&vkey)
             .to_request();
 
@@ -921,7 +1014,7 @@ mod tests {
         let expected =
             compute_hmac(k.as_ref(), challenge.as_bytes()).unwrap(); //#[allow_ci]
         let req = test::TestRequest::get()
-            .uri(&format!("/{API_VERSION}/keys/verify?challenge={challenge}"))
+            .uri(format!("/vX.Y/keys/verify?challenge={challenge}").as_ref())
             .to_request();
         let resp = test::call_service(&app, req).await;
         assert!(resp.status().is_success());
@@ -937,7 +1030,7 @@ mod tests {
         let (new_u, new_v, new_k) =
             prepare_encrypted_keys(key_len, None, uuid, &pubkey);
         let req = test::TestRequest::post()
-            .uri(&format!("/{API_VERSION}/keys/ukey"))
+            .uri("/vX.Y/keys/ukey")
             .set_json(&new_u)
             .to_request();
         let resp = test::call_service(&app, req).await;
@@ -953,7 +1046,7 @@ mod tests {
         };
 
         let req = test::TestRequest::post()
-            .uri(&format!("/{API_VERSION}/keys/vkey"))
+            .uri("/vX.Y/keys/vkey")
             .set_json(&new_v)
             .to_request();
         let resp = test::call_service(&app, req).await;
@@ -972,6 +1065,9 @@ mod tests {
         keys_tx.send((KeyMessage::Shutdown, None)).await.unwrap(); //#[allow_ci]
         payload_tx.send(PayloadMessage::Shutdown).await.unwrap(); //#[allow_ci]
         arbiter.join();
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(quotedata);
     }
 
     #[cfg(feature = "testing")]
@@ -989,16 +1085,17 @@ mod tests {
     #[cfg(feature = "testing")]
     #[actix_rt::test]
     async fn test_pubkey() {
-        let quotedata = web::Data::new(QuoteData::fixture().unwrap()); //#[allow_ci]
-        let mut app =
-            test::init_service(App::new().app_data(quotedata.clone()).route(
-                &format!("/{API_VERSION}/keys/pubkey"),
-                web::get().to(pubkey),
-            ))
-            .await;
+        let (fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+        let quotedata = web::Data::new(fixture);
+        let mut app = test::init_service(
+            App::new()
+                .app_data(quotedata.clone())
+                .route("/vX.Y/keys/pubkey", web::get().to(pubkey)),
+        )
+        .await;
 
         let req = test::TestRequest::get()
-            .uri(&format!("/{API_VERSION}/keys/pubkey"))
+            .uri("/vX.Y/keys/pubkey")
             .to_request();
 
         let resp = test::call_service(&app, req).await;
@@ -1009,5 +1106,8 @@ mod tests {
         assert!(pkey_pub_from_pem(&result.results.pubkey)
             .unwrap() //#[allow_ci]
             .public_eq(&quotedata.pub_key));
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(quotedata);
     }
 }
