@@ -2,8 +2,7 @@
 // Copyright 2021 Keylime Authors
 
 use crate::{
-    common::{EncryptedData, SymmKey},
-    config, crypto,
+    config,
     revocation::{Revocation, RevocationMessage},
     Error, Result,
 };
@@ -11,6 +10,12 @@ use crate::{
 #[cfg(feature = "with-zmq")]
 use crate::revocation::ZmqMessage;
 
+use keylime::crypto::{
+    self,
+    encrypted_data::EncryptedData,
+    symmkey::{KeySet, SymmKey},
+};
+use keylime::global_config;
 use log::*;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
@@ -76,15 +81,17 @@ fn setup_unzipped(
     }
 
     match config.agent.dec_payload_file.as_ref() {
-        "" => Err(Error::Configuration(
-            "The dec_payload_path option was not set".to_string(),
-        )),
+        "" => Err(global_config::KeylimeConfigError::RequiredOption(
+            "dec_payload_path".to_string(),
+        )
+        .into()),
         p => {
             let dec_payload_path = unzipped.join(p);
             match config.agent.enc_keyname.as_ref() {
-                "" => Err(Error::Configuration(
-                    "The enc_keyname option was not set".to_string(),
-                )),
+                "" => Err(global_config::KeylimeConfigError::RequiredOption(
+                    "enc_keyname".to_string(),
+                )
+                .into()),
                 k => {
                     let key_path = unzipped.join(k);
                     fs::create_dir(&unzipped)?;
@@ -322,7 +329,6 @@ mod tests {
         encrypt_aead, pkey_pub_from_pem, rsa_oaep_encrypt,
     };
     use crate::{
-        common::API_VERSION,
         config::KeylimeConfig,
         crypto::{AES_128_KEY_LEN, AES_256_KEY_LEN},
         payloads,
