@@ -5,8 +5,9 @@ use crate::common::JsonWrapper;
 use crate::crypto;
 use crate::serialization::serialize_maybe_base64;
 use crate::{tpm, Error as KeylimeError, QuoteData};
-use actix_web::{web, HttpRequest, HttpResponse, Responder};
+use actix_web::{http, web, HttpRequest, HttpResponse, Responder};
 use base64::{engine::general_purpose, Engine as _};
+use keylime::quote::{Integ, KeylimeQuote};
 use log::*;
 use serde::{Deserialize, Serialize};
 use std::{
@@ -20,37 +21,13 @@ pub struct Ident {
     nonce: String,
 }
 
-#[derive(Deserialize)]
-pub struct Integ {
-    nonce: String,
-    mask: String,
-    partial: String,
-    ima_ml_entry: Option<String>,
-}
-
-#[derive(Serialize, Deserialize, Debug, Default)]
-pub(crate) struct KeylimeQuote {
-    pub quote: String, // 'r' + quote + sig + pcrblob
-    pub hash_alg: String,
-    pub enc_alg: String,
-    pub sign_alg: String,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub pubkey: Option<String>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub ima_measurement_list: Option<String>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub mb_measurement_list: Option<String>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub ima_measurement_list_entry: Option<u64>,
-}
-
 // This is a Quote request from the tenant, which does not check
 // integrity measurement. It should return this data:
 // { QuoteAIK(nonce, 16:H(NK_pub)), NK_pub }
-pub async fn identity(
+async fn identity(
     req: HttpRequest,
     param: web::Query<Ident>,
-    data: web::Data<QuoteData>,
+    data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     // nonce can only be in alphanumerical format
     if !param.nonce.chars().all(char::is_alphanumeric) {
@@ -136,10 +113,10 @@ pub async fn identity(
 // by the mask. It should return this data:
 // { QuoteAIK(nonce, 16:H(NK_pub), xi:yi), NK_pub}
 // where xi:yi are additional PCRs to be included in the quote.
-pub async fn integrity(
+async fn integrity(
     req: HttpRequest,
     param: web::Query<Integ>,
-    data: web::Data<QuoteData>,
+    data: web::Data<QuoteData<'_>>,
 ) -> impl Responder {
     // nonce, mask can only be in alphanumerical format
     if !param.nonce.chars().all(char::is_alphanumeric) {
@@ -336,28 +313,67 @@ pub async fn integrity(
     HttpResponse::Ok().json(response)
 }
 
+/// Handles the default case for the /quotes scope
+async fn quotes_default(req: HttpRequest) -> impl Responder {
+    let error;
+    let response;
+    let message;
+
+    match req.head().method {
+        http::Method::GET => {
+            error = 400;
+            message = "URI not supported, only /identity and /integrity are supported for GET in /quotes/ interface";
+            response = HttpResponse::BadRequest()
+                .json(JsonWrapper::error(error, message));
+        }
+        _ => {
+            error = 405;
+            message = "Method is not supported in /quotes/ interface";
+            response = HttpResponse::MethodNotAllowed()
+                .insert_header(http::header::Allow(vec![http::Method::GET]))
+                .json(JsonWrapper::error(error, message));
+        }
+    };
+
+    warn!(
+        "{} returning {} response. {}",
+        req.head().method,
+        error,
+        message
+    );
+
+    response
+}
+
+/// Configure the endpoints for the /quotes scope
+pub(crate) fn configure_quotes_endpoints(cfg: &mut web::ServiceConfig) {
+    _ = cfg
+        .service(web::resource("/identity").route(web::get().to(identity)))
+        .service(web::resource("/integrity").route(web::get().to(integrity)))
+        .default_service(web::to(quotes_default));
+}
+
 #[cfg(feature = "testing")]
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::common::API_VERSION;
     use actix_web::{test, web, App};
     use keylime::{crypto::testing::pkey_pub_from_pem, tpm};
+    use serde_json::{json, Value};
 
     #[actix_rt::test]
     async fn test_identity() {
-        let quotedata = web::Data::new(QuoteData::fixture().unwrap()); //#[allow_ci]
-        let mut app =
-            test::init_service(App::new().app_data(quotedata.clone()).route(
-                &format!("/{API_VERSION}/quotes/identity"),
-                web::get().to(identity),
-            ))
-            .await;
+        let (fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+        let quotedata = web::Data::new(fixture);
+        let mut app = test::init_service(
+            App::new()
+                .app_data(quotedata.clone())
+                .route("/vX.Y/quotes/identity", web::get().to(identity)),
+        )
+        .await;
 
         let req = test::TestRequest::get()
-            .uri(&format!(
-                "/{API_VERSION}/quotes/identity?nonce=1234567890ABCDEFHIJ",
-            ))
+            .uri("/vX.Y/quotes/identity?nonce=1234567890ABCDEFHIJ")
             .to_request();
 
         let resp = test::call_service(&app, req).await;
@@ -377,28 +393,33 @@ mod tests {
 
         let mut context = quotedata.tpmcontext.lock().unwrap(); //#[allow_ci]
         tpm::testing::check_quote(
-            context.as_mut(),
+            &mut context,
             quotedata.ak_handle,
             &result.results.quote,
             b"1234567890ABCDEFHIJ",
         )
         .expect("unable to verify quote");
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(context);
+        drop(quotedata);
     }
 
     #[actix_rt::test]
     async fn test_integrity_pre() {
-        let quotedata = web::Data::new(QuoteData::fixture().unwrap()); //#[allow_ci]
-        let mut app =
-            test::init_service(App::new().app_data(quotedata.clone()).route(
-                &format!("/{API_VERSION}/quotes/integrity"),
-                web::get().to(integrity),
-            ))
-            .await;
+        let (fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+        let quotedata = web::Data::new(fixture);
+        let mut app = test::init_service(
+            App::new()
+                .app_data(quotedata.clone())
+                .route("vX.Y/quotes/integrity", web::get().to(integrity)),
+        )
+        .await;
 
         let req = test::TestRequest::get()
-            .uri(&format!(
-                "/{API_VERSION}/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=0",
-            ))
+            .uri(
+                "/vX.Y/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=0",
+            )
             .to_request();
 
         let resp = test::call_service(&app, req).await;
@@ -429,7 +450,7 @@ mod tests {
 
                     let mut context = quotedata.tpmcontext.lock().unwrap(); //#[allow_ci]
                     tpm::testing::check_quote(
-                        context.as_mut(),
+                        &mut context,
                         quotedata.ak_handle,
                         &result.results.quote,
                         b"1234567890ABCDEFHIJ",
@@ -441,22 +462,26 @@ mod tests {
         } else {
             panic!("IMA file was None"); //#[allow_ci]
         }
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(quotedata);
     }
 
     #[actix_rt::test]
     async fn test_integrity_post() {
-        let quotedata = web::Data::new(QuoteData::fixture().unwrap()); //#[allow_ci]
-        let mut app =
-            test::init_service(App::new().app_data(quotedata.clone()).route(
-                &format!("/{API_VERSION}/quotes/integrity"),
-                web::get().to(integrity),
-            ))
-            .await;
+        let (fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+        let quotedata = web::Data::new(fixture);
+        let mut app = test::init_service(
+            App::new()
+                .app_data(quotedata.clone())
+                .route("/vX.Y/quotes/integrity", web::get().to(integrity)),
+        )
+        .await;
 
         let req = test::TestRequest::get()
-            .uri(&format!(
-                "/{API_VERSION}/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=1",
-            ))
+            .uri(
+                "/vX.Y/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=1",
+            )
             .to_request();
 
         let resp = test::call_service(&app, req).await;
@@ -488,31 +513,36 @@ mod tests {
 
         let mut context = quotedata.tpmcontext.lock().unwrap(); //#[allow_ci]
         tpm::testing::check_quote(
-            context.as_mut(),
+            &mut context,
             quotedata.ak_handle,
             &result.results.quote,
             b"1234567890ABCDEFHIJ",
         )
         .expect("unable to verify quote");
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(context);
+        drop(quotedata);
     }
 
     #[actix_rt::test]
     async fn test_missing_ima_file() {
-        let mut quotedata = QuoteData::fixture().unwrap(); //#[allow_ci]
-                                                           // Remove the IMA log file from the context
-        quotedata.ima_ml_file = None;
-        let data = web::Data::new(quotedata);
-        let mut app =
-            test::init_service(App::new().app_data(data.clone()).route(
-                &format!("/{API_VERSION}/quotes/integrity"),
-                web::get().to(integrity),
-            ))
-            .await;
+        let (mut fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+
+        // Remove the IMA log file from the context
+        fixture.ima_ml_file = None;
+        let quotedata = web::Data::new(fixture);
+        let mut app = test::init_service(
+            App::new()
+                .app_data(quotedata.clone())
+                .route("/vX.Y/quotes/integrity", web::get().to(integrity)),
+        )
+        .await;
 
         let req = test::TestRequest::get()
-            .uri(&format!(
-                "/{API_VERSION}/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=0",
-            ))
+            .uri(
+                "/vX.Y/quotes/integrity?nonce=1234567890ABCDEFHIJ&mask=0x408000&partial=0",
+            )
             .to_request();
 
         let resp = test::call_service(&app, req).await;
@@ -522,5 +552,41 @@ mod tests {
             test::read_body_json(resp).await;
         assert!(result.results.ima_measurement_list.is_none());
         assert!(result.results.ima_measurement_list_entry.is_none());
+
+        // Explicitly drop QuoteData to cleanup keys
+        drop(quotedata);
+    }
+
+    #[actix_rt::test]
+    async fn test_keys_default() {
+        let mut app = test::init_service(
+            App::new().service(web::resource("/").to(quotes_default)),
+        )
+        .await;
+
+        let req = test::TestRequest::get().uri("/").to_request();
+
+        let resp = test::call_service(&app, req).await;
+        assert!(resp.status().is_client_error());
+
+        let result: JsonWrapper<Value> = test::read_body_json(resp).await;
+
+        assert_eq!(result.results, json!({}));
+        assert_eq!(result.code, 400);
+
+        let req = test::TestRequest::delete().uri("/").to_request();
+
+        let resp = test::call_service(&app, req).await;
+        assert!(resp.status().is_client_error());
+
+        let headers = resp.headers();
+
+        assert!(headers.contains_key("allow"));
+        assert_eq!(headers.get("allow").unwrap().to_str().unwrap(), "GET"); //#[allow_ci]
+
+        let result: JsonWrapper<Value> = test::read_body_json(resp).await;
+
+        assert_eq!(result.results, json!({}));
+        assert_eq!(result.code, 405);
     }
 }
