@@ -5,8 +5,8 @@
 use actix_web::rt;
 use crate::config::{AgentConfig, KeylimeConfig};
 use crate::crypto;
-use crate::error::*;
 use crate::secure_mount;
+use keylime::keylime_error::*;
 use keylime::list_parser::parse_list;
 use log::*;
 use serde::{Deserialize, Serialize};
@@ -51,6 +51,7 @@ pub(crate) enum RevocationMessage {
 /// The lookup goes in the following order:
 /// 1. Look for pre-installed action
 /// 2. Look for the action in the tenant-provided initial payload
+///
 /// Then, if python revocation actions are allowed:
 /// 3. Look for pre-installed Python action
 /// 4. Look for the Python action in the tenant-provided initial payload
@@ -162,7 +163,7 @@ pub(crate) fn run_action(
         }
         Err(err) => {
             fs::remove_file(json_path)?;
-            return Err(err.try_into()?);
+            return Err(err.into());
         }
     };
 
